# -*- coding: utf-8 -*-

from generate_features_labels import *
from filter_features_matrix import *

import cPickle
import xgboost
import argparse

def train_model_save(load_path,save_path):
	## Load train.txt and build features matrix
	print "Generating customer transaction list..."
	cstm_lst = get_customers_lst(load_path)
	print "Customer list generated"
	print "Building matrix features and labels vectors..."
	X, y, custom_pos = build_X_y_from_custom_iter_ch_V3(
		cstm_lst,
		len(cstm_lst),
		add_bag_of_prods = True,
		add_bop_abs_dates = False,
		add_bop_rel_dates = False,
		add_last_prods = 3,
		add_lst_prd_abs_dates = True,
		add_lst_prd_rel_dates = True,
		scale_abs_dates_equal = False,
		scale_rel_dates_equal = False,
		orig_date = datetime.datetime(1950,1,1),
		abs_axis_norm = 0,
		rel_axis_norm = 0,
		rel_alpha = 24*3600,
		debug = False,
	)
	print "Features matrix and labels built"

	## Delete customers with only one product
	print "Filtering customers..."
	with open("./auxiliary_data/one_prod_cstms.p","rb") as f:
		one_prod_cstms = cPickle.load(f)
	X,y = delete_rows_with_no_products_v2(X,y,custom_pos,one_prod_cstms)

	## Filter out classes with too few samples
	X, y = delete_few_samples_costumts(X,y,min_samples=10)
	print "Done"

	## Train model and save it
	print "Training model (this will take a while)..."
	model = xgboost.XGBClassifier(
			n_estimators = 300,
			max_depth = 4,
			gamma = 0.3,
			learning_rate = 0.2
			)
	model.fit(X,y)
	print "Done"

	print "Saving model..."
	with open(save_path,"wb") as f:
		cPickle.dump(model,f,cPickle.HIGHEST_PROTOCOL)
	print "Model saved in %s\nEverything finished!"%save_path

if __name__ == "__main__":
	load_path = "./data/train2.txt"
	save_path = "./models/model.p"

	parser = argparse.ArgumentParser(description='Train an XGboost model with a train file.')
	parser.add_argument('--train', type=str,
	                   help='Path of the train file (default: %s)'%load_path,
	                   default=load_path)
	parser.add_argument('--save', type=str,
	                   help='Path to save the model (default: %s)'%save_path,
	                   default=save_path)
	args = parser.parse_args()

	load_path = args.train
	save_path = args.save

	train_model_save(load_path,save_path)